use std::{ffi::OsString, fs, net::TcpListener, process};

use ctrlc;
use http::responses::{Body, Response};

mod handlers;
mod http;
mod icons;

fn main() {
    println!("luciders starting...");

    let listener =
        TcpListener::bind("[::]:7878").unwrap_or_else(|_| fatal("Failed to bind to address"));

    ctrlc::set_handler(move || {
        println!("Shutting down...");
        process::exit(0);
    })
    .unwrap_or_else(|_| fatal("Failed to set termination signal handler"));

    let icons = match icons::Icons::build() {
        Ok(icons) => icons,
        Err(e) => fatal(e),
    };

    let mut handlers = handlers::Handlers::new();

    handlers.add_handler("/", |req| {
        if req.method != "GET" {
            return Response::new(req, 405, Body::Static("Method Not Allowed"));
        }

        Response::new(req, 200, Body::Static("OK - luciders is running"))
    });

    handlers.add_handler("/icons/[icon].svg", move |req| {
        if req.method != "GET" {
            return Response::new(req, 405, Body::Static("Method Not Allowed"));
        }

        let icon_filename = match req.url.get_path_segment(1) {
            Some(icon) => icon,
            None => return Response::new(req, 404, Body::Static("Icon Not Found")),
        };

        let icon_name = &icon_filename[..icon_filename.len() - icons::ICON_FILE_EXTENSION_LEN];

        if !icons.has_iconname(icon_name) {
            return Response::new(req, 404, Body::Static("Icon Not Found"));
        }

        let icon = match icons.get_icon(icon_name) {
            Some(icon) => icon,
            None => return Response::new(req, 500, Body::Static("Failed to read icon")),
        };

        return Response::new(req, 200, Body::Bytes(icon, "image/svg+xml"));
    });

    handlers.bind(listener);
}

fn fatal(msg: &'static str) -> ! {
    eprintln!("[FATAL] {}", msg);
    process::exit(1);
}

fn error(msg: &'static str) {
    eprintln!("[ERROR] {}", msg);
}
